# Copyright (C) 2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import re
import click
import logging

from swh.scheduler import get_scheduler
from swh.scheduler.utils import create_task_dict
from swh.indexer.indexer import OriginIndexer

from swh.model.hashutil import hash_to_hex


class OriginHeadIndexer(OriginIndexer):
    """Origin-level indexer.

    This indexer is in charge of looking up the revision that acts as the
    "head" of an origin.

    In git, this is usually the commit pointed to by the 'master' branch."""

    ADDITIONAL_CONFIG = {
        'tools': ('dict', {
            'name': 'origin-metadata',
            'version': '0.0.1',
            'configuration': {},
        }),
        'tasks': ('dict', {
            'revision_metadata': 'revision_metadata',
            'origin_intrinsic_metadata': 'origin_metadata',
        })
    }

    CONFIG_BASE_FILENAME = 'indexer/origin_head'

    def filter(self, ids):
        yield from ids

    def persist_index_computations(self, results, policy_update):
        """Do nothing. The indexer's results are not persistent, they
        should only be piped to another indexer."""
        pass

    def next_step(self, results, task):
        """Once the head is found, call the RevisionMetadataIndexer
        on these revisions, then call the OriginMetadataIndexer with
        both the origin_id and the revision metadata, so it can copy the
        revision metadata to the origin's metadata.

        Args:
            results (Iterable[dict]): Iterable of return values from `index`.

        """
        super().next_step(results, task)
        revision_metadata_task = self.config['tasks']['revision_metadata']
        origin_intrinsic_metadata_task = self.config['tasks'][
            'origin_intrinsic_metadata']
        if revision_metadata_task is None and \
                origin_intrinsic_metadata_task is None:
            return
        assert revision_metadata_task is not None
        assert origin_intrinsic_metadata_task is not None

        # Second task to run after this one: copy the revision's metadata
        # to the origin
        sub_task = create_task_dict(
            origin_intrinsic_metadata_task,
            'oneshot',
            origin_head={
                str(result['origin_id']):
                    hash_to_hex(result['revision_id'])
                for result in results},
            policy_update='update-dups',
            )
        del sub_task['next_run']  # Not json-serializable

        # First task to run after this one: index the metadata of the
        # revision
        task = create_task_dict(
            revision_metadata_task,
            'oneshot',
            ids=[hash_to_hex(res['revision_id']) for res in results],
            policy_update='update-dups',
            next_step=sub_task,
            )
        if getattr(self, 'scheduler', None):
            scheduler = self.scheduler
        else:
            scheduler = get_scheduler(**self.config['scheduler'])
        scheduler.create_tasks([task])

    # Dispatch

    def index(self, origin):
        origin_id = origin['id']
        latest_snapshot = self.storage.snapshot_get_latest(origin_id)
        method = getattr(self, '_try_get_%s_head' % origin['type'], None)
        if method is None:
            method = self._try_get_head_generic
        rev_id = method(latest_snapshot)
        if rev_id is None:
            return None
        result = {
                'origin_id': origin_id,
                'revision_id': rev_id,
                }
        return result

    # VCSs

    def _try_get_vcs_head(self, snapshot):
        try:
            if isinstance(snapshot, dict):
                branches = snapshot['branches']
                if branches[b'HEAD']['target_type'] == 'revision':
                    return branches[b'HEAD']['target']
        except KeyError:
            return None

    _try_get_hg_head = _try_get_git_head = _try_get_vcs_head

    # Tarballs

    _archive_filename_re = re.compile(
            rb'^'
            rb'(?P<pkgname>.*)[-_]'
            rb'(?P<version>[0-9]+(\.[0-9])*)'
            rb'(?P<preversion>[-+][a-zA-Z0-9.~]+?)?'
            rb'(?P<extension>(\.[a-zA-Z0-9]+)+)'
            rb'$')

    @classmethod
    def _parse_version(cls, filename):
        """Extracts the release version from an archive filename,
        to get an ordering whose maximum is likely to be the last
        version of the software

        >>> OriginHeadIndexer._parse_version(b'foo')
        (-inf,)
        >>> OriginHeadIndexer._parse_version(b'foo.tar.gz')
        (-inf,)
        >>> OriginHeadIndexer._parse_version(b'gnu-hello-0.0.1.tar.gz')
        (0, 0, 1, 0)
        >>> OriginHeadIndexer._parse_version(b'gnu-hello-0.0.1-beta2.tar.gz')
        (0, 0, 1, -1, 'beta2')
        >>> OriginHeadIndexer._parse_version(b'gnu-hello-0.0.1+foobar.tar.gz')
        (0, 0, 1, 1, 'foobar')
        """
        res = cls._archive_filename_re.match(filename)
        if res is None:
            return (float('-infinity'),)
        version = [int(n) for n in res.group('version').decode().split('.')]
        if res.group('preversion') is None:
            version.append(0)
        else:
            preversion = res.group('preversion').decode()
            if preversion.startswith('-'):
                version.append(-1)
                version.append(preversion[1:])
            elif preversion.startswith('+'):
                version.append(1)
                version.append(preversion[1:])
            else:
                assert False, res.group('preversion')
        return tuple(version)

    def _try_get_ftp_head(self, snapshot):
        archive_names = list(snapshot['branches'])
        max_archive_name = max(archive_names, key=self._parse_version)
        r = self._try_resolve_target(snapshot['branches'], max_archive_name)
        return r

    # Generic

    def _try_get_head_generic(self, snapshot):
        # Works on 'deposit', 'svn', and 'pypi'.
        try:
            if isinstance(snapshot, dict):
                branches = snapshot['branches']
        except KeyError:
            return None
        else:
            return (
                    self._try_resolve_target(branches, b'HEAD') or
                    self._try_resolve_target(branches, b'master')
                    )

    def _try_resolve_target(self, branches, target_name):
        try:
            target = branches[target_name]
            while target['target_type'] == 'alias':
                target = branches[target['target']]
            if target['target_type'] == 'revision':
                return target['target']
            elif target['target_type'] == 'content':
                return None  # TODO
            elif target['target_type'] == 'directory':
                return None  # TODO
            elif target['target_type'] == 'release':
                return None  # TODO
            else:
                assert False
        except KeyError:
            return None


@click.command()
@click.option('--origins', '-i',
              help='Origins to lookup, in the "type+url" format',
              multiple=True)
def main(origins):
    rev_metadata_indexer = OriginHeadIndexer()
    rev_metadata_indexer.run(origins)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    main()



# Copyright (C) 2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import time
import unittest

from celery import task
from swh.model.hashutil import hash_to_bytes
from swh.storage.in_memory import Storage

from swh.indexer.metadata import (
    OriginMetadataIndexer, RevisionMetadataIndexer
)

from swh.indexer.storage.in_memory import IndexerStorage
from swh.objstorage.objstorage_in_memory import InMemoryObjStorage

from swh.scheduler.tests.scheduler_testing import SchedulerTestFixture
from .test_utils import (
    BASE_TEST_CONFIG, fill_storage, fill_obj_storage
)
from .test_origin_head import OriginHeadTestIndexer
from .test_metadata import ContentMetadataTestIndexer


class RevisionMetadataTestIndexer(RevisionMetadataIndexer):
    """Specific indexer whose configuration is enough to satisfy the
       indexing tests.
    """
    ContentMetadataIndexer = ContentMetadataTestIndexer

    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'tools': {
                'name': 'swh-metadata-detector',
                'version': '0.0.2',
                'configuration': {
                    'type': 'local',
                    'context': 'NpmMapping'
                }
            }
        }


@task
def revision_metadata_test_task(*args, **kwargs):
    indexer = RevisionMetadataTestIndexer()
    indexer.run(*args, **kwargs)
    return indexer.results


class OriginMetadataTestIndexer(OriginMetadataIndexer):
    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'tools': []
        }


@task
def origin_intrinsic_metadata_test_task(*args, **kwargs):
    indexer = OriginMetadataTestIndexer()
    indexer.run(*args, **kwargs)
    return indexer.results


class OriginHeadTestIndexer(OriginHeadTestIndexer):
    def prepare(self):
        super().prepare()
        self.config['tasks'] = {
            'revision_metadata': 'revision_metadata_test_task',
            'origin_intrinsic_metadata': 'origin_intrinsic_metadata_test_task',
        }


class TestOriginMetadata(SchedulerTestFixture, unittest.TestCase):
    def setUp(self):
        super().setUp()
        self.maxDiff = None
        self.add_scheduler_task_type(
            'revision_metadata_test_task',
            'swh.indexer.tests.test_origin_metadata.'
            'revision_metadata_test_task')
        self.add_scheduler_task_type(
            'origin_intrinsic_metadata_test_task',
            'swh.indexer.tests.test_origin_metadata.'
            'origin_intrinsic_metadata_test_task')
        RevisionMetadataTestIndexer.scheduler = self.scheduler

    def tearDown(self):
        del RevisionMetadataTestIndexer.scheduler
        super().tearDown()

    @unittest.mock.patch('swh.indexer.storage.in_memory.IndexerStorage')
    @unittest.mock.patch('swh.storage.in_memory.Storage')
    def test_pipeline(self, storage_mock, idx_storage_mock):
        # Always returns the same instance of the idx storage, because
        # this function is called by each of the three indexers.
        objstorage = InMemoryObjStorage()
        storage = Storage()
        idx_storage = IndexerStorage()

        storage_mock.return_value = storage
        idx_storage_mock.return_value = idx_storage

        fill_obj_storage(objstorage)
        fill_storage(storage)

        # TODO: find a better way to share the ContentMetadataIndexer use
        # the same objstorage instance.
        import swh.objstorage
        old_inmem_objstorage = swh.objstorage._STORAGE_CLASSES['memory']
        swh.objstorage._STORAGE_CLASSES['memory'] = lambda: objstorage
        try:
            indexer = OriginHeadTestIndexer()
            indexer.scheduler = self.scheduler
            indexer.run(["git+https://github.com/librariesio/yarn-parser"])

            self.run_ready_tasks()  # Run the first task
            # Give it time to complete and schedule the 2nd one
            time.sleep(0.1)
            self.run_ready_tasks()  # Run the second task
        finally:
            swh.objstorage._STORAGE_CLASSES['memory'] = old_inmem_objstorage

        origin = storage.origin_get({
            'type': 'git',
            'url': 'https://github.com/librariesio/yarn-parser'})
        rev_id = hash_to_bytes('8dbb6aeb036e7fd80664eb8bfd1507881af1ba9f')

        metadata = {
            '@context': 'https://doi.org/10.5063/schema/codemeta-2.0',
            'url':
                'https://github.com/librariesio/yarn-parser#readme',
            'codeRepository':
                'git+git+https://github.com/librariesio/yarn-parser.git',
            'author': [{
                'type': 'Person',
                'name': 'Andrew Nesbitt'
            }],
            'license': 'https://spdx.org/licenses/AGPL-3.0',
            'version': '1.0.0',
            'description':
                'Tiny web service for parsing yarn.lock files',
            'issueTracker':
                'https://github.com/librariesio/yarn-parser/issues',
            'name': 'yarn-parser',
            'keywords': ['yarn', 'parse', 'lock', 'dependencies'],
        }
        rev_metadata = {
            'id': rev_id,
            'translated_metadata': metadata,
        }
        origin_metadata = {
            'origin_id': origin['id'],
            'from_revision': rev_id,
            'metadata': metadata,
        }

        results = list(indexer.idx_storage.revision_metadata_get([rev_id]))
        for result in results:
            del result['tool']
        self.assertEqual(results, [rev_metadata])

        results = list(indexer.idx_storage.origin_intrinsic_metadata_get([
            origin['id']]))
        for result in results:
            del result['tool']
        self.assertEqual(results, [origin_metadata])



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import unittest

from unittest.mock import patch

from swh.indexer.fossology_license import (
    FossologyLicenseIndexer, FossologyLicenseRangeIndexer,
    compute_license
)

from swh.indexer.tests.test_utils import (
    SHA1_TO_LICENSES, CommonContentIndexerTest, CommonContentIndexerRangeTest,
    CommonIndexerWithErrorsTest, CommonIndexerNoTool, NoDiskIndexer,
    BASE_TEST_CONFIG, fill_storage, fill_obj_storage
)


class BasicTest(unittest.TestCase):
    @patch('swh.indexer.fossology_license.subprocess')
    def test_compute_license(self, mock_subprocess):
        """Computing licenses from a raw content should return results

        """
        for path, intermediary_result, output in [
                (b'some/path', None,
                 []),
                (b'some/path/2', [],
                 []),
                (b'other/path', ' contains license(s) GPL,AGPL',
                 ['GPL', 'AGPL'])]:
            mock_subprocess.check_output.return_value = intermediary_result

            actual_result = compute_license(path, log=None)

            self.assertEqual(actual_result, {
                'licenses': output,
                'path': path,
            })


class InjectLicenseIndexer:
    """Override license computations.

    """
    def compute_license(self, path, log=None):
        """path is the content identifier

        """
        if isinstance(id, bytes):
            path = path.decode('utf-8')
        return {
            'licenses': SHA1_TO_LICENSES.get(path)
        }


class FossologyLicenseTestIndexer(
        NoDiskIndexer, InjectLicenseIndexer, FossologyLicenseIndexer):
    """Specific fossology license whose configuration is enough to satisfy
       the indexing checks.

    """
    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'workdir': '/nowhere',
            'tools': {
                'name': 'nomos',
                'version': '3.1.0rc2-31-ga2cbb8c',
                'configuration': {
                    'command_line': 'nomossa <filepath>',
                },
            },
        }


class TestFossologyLicenseIndexer(CommonContentIndexerTest, unittest.TestCase):
    """Language indexer test scenarios:

    - Known sha1s in the input list have their data indexed
    - Unknown sha1 in the input list are not indexed

    """

    def get_indexer_results(self, ids):
        yield from self.idx_storage.content_fossology_license_get(ids)

    def setUp(self):
        super().setUp()
        self.indexer = FossologyLicenseTestIndexer()
        self.idx_storage = self.indexer.idx_storage
        fill_storage(self.indexer.storage)
        fill_obj_storage(self.indexer.objstorage)

        self.id0 = '01c9379dfc33803963d07c1ccc748d3fe4c96bb5'
        self.id1 = '688a5ef812c53907562fe379d4b3851e69c7cb15'
        self.id2 = 'da39a3ee5e6b4b0d3255bfef95601890afd80709'  # empty content

        tool = {k.replace('tool_', ''): v
                for (k, v) in self.indexer.tool.items()}
        # then
        self.expected_results = {
            self.id0: {
                'tool': tool,
                'licenses': SHA1_TO_LICENSES[self.id0],
            },
            self.id1: {
                'tool': tool,
                'licenses': SHA1_TO_LICENSES[self.id1],
            },
            self.id2: {
                'tool': tool,
                'licenses': SHA1_TO_LICENSES[self.id2],
            }
        }


class FossologyLicenseRangeIndexerTest(
        NoDiskIndexer, InjectLicenseIndexer, FossologyLicenseRangeIndexer):
    """Testing the range indexer on fossology license.

    """
    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'workdir': '/nowhere',
            'tools': {
                'name': 'nomos',
                'version': '3.1.0rc2-31-ga2cbb8c',
                'configuration': {
                    'command_line': 'nomossa <filepath>',
                },
            },
            'write_batch_size': 100,
        }


class TestFossologyLicenseRangeIndexer(
        CommonContentIndexerRangeTest, unittest.TestCase):
    """Range Fossology License Indexer tests.

    - new data within range are indexed
    - no data outside a range are indexed
    - with filtering existing indexed data prior to compute new index
    - without filtering existing indexed data prior to compute new index

    """
    def setUp(self):
        super().setUp()
        self.indexer = FossologyLicenseRangeIndexerTest()
        fill_storage(self.indexer.storage)
        fill_obj_storage(self.indexer.objstorage)

        self.id0 = '01c9379dfc33803963d07c1ccc748d3fe4c96bb5'
        self.id1 = '02fb2c89e14f7fab46701478c83779c7beb7b069'
        self.id2 = '103bc087db1d26afc3a0283f38663d081e9b01e6'
        tool_id = self.indexer.tool['id']
        self.expected_results = {
            self.id0: {
                'id': self.id0,
                'indexer_configuration_id': tool_id,
                'licenses': SHA1_TO_LICENSES[self.id0]
            },
            self.id1: {
                'id': self.id1,
                'indexer_configuration_id': tool_id,
                'licenses': SHA1_TO_LICENSES[self.id1]
            },
            self.id2: {
                'id': self.id2,
                'indexer_configuration_id': tool_id,
                'licenses': SHA1_TO_LICENSES[self.id2]
            }
        }


class FossologyLicenseIndexerUnknownToolTestStorage(
        CommonIndexerNoTool, FossologyLicenseTestIndexer):
    """Fossology license indexer with wrong configuration"""


class FossologyLicenseRangeIndexerUnknownToolTestStorage(
        CommonIndexerNoTool, FossologyLicenseRangeIndexerTest):
    """Fossology license range indexer with wrong configuration"""


class TestFossologyLicenseIndexersErrors(
        CommonIndexerWithErrorsTest, unittest.TestCase):
    """Test the indexer raise the right errors when wrongly initialized"""
    Indexer = FossologyLicenseIndexerUnknownToolTestStorage
    RangeIndexer = FossologyLicenseRangeIndexerUnknownToolTestStorage



# Copyright (C) 2016-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import logging

from swh.scheduler.task import Task as SchedulerTask

from .mimetype import MimetypeIndexer, MimetypeRangeIndexer
from .language import LanguageIndexer
from .ctags import CtagsIndexer
from .fossology_license import (
    FossologyLicenseIndexer, FossologyLicenseRangeIndexer
)
from .rehash import RecomputeChecksums
from .metadata import RevisionMetadataIndexer, OriginMetadataIndexer
from .origin_head import OriginHeadIndexer

logging.basicConfig(level=logging.INFO)


class Task(SchedulerTask):
    """Task whose results is needed for other computations.

    """
    def run_task(self, *args, **kwargs):
        indexer = self.Indexer().run(*args, **kwargs)
        if hasattr(indexer, 'results'):  # indexer tasks
            return indexer.results
        return indexer


class StatusTask(SchedulerTask):
    """Task which returns a status either eventful or uneventful.

    """
    def run_task(self, *args, **kwargs):
        results = self.Indexer().run(*args, **kwargs)
        return {'status': 'eventful' if results else 'uneventful'}


class RevisionMetadata(Task):
    task_queue = 'swh_indexer_revision_metadata'

    serializer = 'msgpack'

    Indexer = RevisionMetadataIndexer


class OriginMetadata(Task):
    task_queue = 'swh_indexer_origin_intrinsic_metadata'

    Indexer = OriginMetadataIndexer


class OriginHead(Task):
    task_queue = 'swh_indexer_origin_head'

    Indexer = OriginHeadIndexer


class ContentMimetype(StatusTask):
    """Compute (mimetype, encoding) on a list of sha1s' content.

    """
    task_queue = 'swh_indexer_content_mimetype'
    Indexer = MimetypeIndexer


class ContentRangeMimetype(StatusTask):
    """Compute (mimetype, encoding) on a range of sha1s.

    """
    task_queue = 'swh_indexer_content_mimetype_range'
    Indexer = MimetypeRangeIndexer


class ContentLanguage(Task):
    """Task which computes the language from the sha1's content.

    """
    task_queue = 'swh_indexer_content_language'

    Indexer = LanguageIndexer


class Ctags(Task):
    """Task which computes ctags from the sha1's content.

    """
    task_queue = 'swh_indexer_content_ctags'

    Indexer = CtagsIndexer


class ContentFossologyLicense(Task):
    """Compute fossology licenses on a list of sha1s' content.

    """
    task_queue = 'swh_indexer_content_fossology_license'
    Indexer = FossologyLicenseIndexer


class ContentRangeFossologyLicense(StatusTask):
    """Compute fossology license on a range of sha1s.

    """
    task_queue = 'swh_indexer_content_fossology_license_range'
    Indexer = FossologyLicenseRangeIndexer


class RecomputeChecksums(Task):
    """Task which recomputes hashes and possibly new ones.

    """
    task_queue = 'swh_indexer_content_rehash'

    Indexer = RecomputeChecksums



# Copyright (C) 2017  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import os
import re
import abc
import json
import logging
import email.parser

import xmltodict

from swh.indexer.codemeta import CROSSWALK_TABLE, SCHEMA_URI
from swh.indexer.codemeta import compact, expand


MAPPINGS = {}


def register_mapping(cls):
    MAPPINGS[cls.__name__] = cls()
    return cls


class BaseMapping(metaclass=abc.ABCMeta):
    """Base class for mappings to inherit from

    To implement a new mapping:

    - inherit this class
    - override translate function
    """
    def __init__(self):
        self.log = logging.getLogger('%s.%s' % (
            self.__class__.__module__,
            self.__class__.__name__))

    @abc.abstractmethod
    def detect_metadata_files(self, files):
        """
        Detects files potentially containing metadata

        Args:
            file_entries (list): list of files

        Returns:
            list: list of sha1 (possibly empty)
        """
        pass

    @abc.abstractmethod
    def translate(self, file_content):
        pass

    def normalize_translation(self, metadata):
        return compact(metadata)


class SingleFileMapping(BaseMapping):
    """Base class for all mappings that use a single file as input."""

    @property
    @abc.abstractmethod
    def filename(self):
        """The .json file to extract metadata from."""
        pass

    def detect_metadata_files(self, file_entries):
        for entry in file_entries:
            if entry['name'] == self.filename:
                return [entry['sha1']]
        return []


class DictMapping(BaseMapping):
    """Base class for mappings that take as input a file that is mostly
    a key-value store (eg. a shallow JSON dict)."""

    @property
    @abc.abstractmethod
    def mapping(self):
        """A translation dict to map dict keys into a canonical name."""
        pass

    def translate_dict(self, content_dict, *, normalize=True):
        """
        Translates content  by parsing content from a dict object
        and translating with the appropriate mapping

        Args:
            content_dict (dict): content dict to translate

        Returns:
            dict: translated metadata in json-friendly form needed for
            the indexer

        """
        translated_metadata = {'@type': SCHEMA_URI + 'SoftwareSourceCode'}
        for k, v in content_dict.items():
            # First, check if there is a specific translation
            # method for this key
            translation_method = getattr(
                self, 'translate_' + k.replace('-', '_'), None)
            if translation_method:
                translation_method(translated_metadata, v)
            elif k in self.mapping:
                # if there is no method, but the key is known from the
                # crosswalk table

                # if there is a normalization method, use it on the value
                normalization_method = getattr(
                    self, 'normalize_' + k.replace('-', '_'), None)
                if normalization_method:
                    v = normalization_method(v)

                # set the translation metadata with the normalized value
                translated_metadata[self.mapping[k]] = v
        if normalize:
            return self.normalize_translation(translated_metadata)
        else:
            return translated_metadata


class JsonMapping(DictMapping, SingleFileMapping):
    """Base class for all mappings that use a JSON file as input."""

    def translate(self, raw_content):
        """
        Translates content by parsing content from a bytestring containing
        json data and translating with the appropriate mapping

        Args:
            raw_content (bytes): raw content to translate

        Returns:
            dict: translated metadata in json-friendly form needed for
            the indexer

        """
        try:
            raw_content = raw_content.decode()
        except UnicodeDecodeError:
            self.log.warning('Error unidecoding %r', raw_content)
            return
        try:
            content_dict = json.loads(raw_content)
        except json.JSONDecodeError:
            self.log.warning('Error unjsoning %r' % raw_content)
            return
        return self.translate_dict(content_dict)


@register_mapping
class NpmMapping(JsonMapping):
    """
    dedicated class for NPM (package.json) mapping and translation
    """
    mapping = CROSSWALK_TABLE['NodeJS']
    filename = b'package.json'

    _schema_shortcuts = {
            'github': 'https://github.com/',
            'gist': 'https://gist.github.com/',
            'bitbucket': 'https://bitbucket.org/',
            'gitlab': 'https://gitlab.com/',
            }

    def normalize_repository(self, d):
        """https://docs.npmjs.com/files/package.json#repository"""
        if isinstance(d, dict):
            url = '{type}+{url}'.format(**d)
        elif isinstance(d, str):
            if '://' in d:
                url = d
            elif ':' in d:
                (schema, rest) = d.split(':', 1)
                if schema in self._schema_shortcuts:
                    url = self._schema_shortcuts[schema] + rest
                else:
                    return None
            else:
                url = self._schema_shortcuts['github'] + d

        else:
            return None

        return {'@id': url}

    def normalize_bugs(self, d):
        return {'@id': '{url}'.format(**d)}

    _parse_author = re.compile(r'^ *'
                               r'(?P<name>.*?)'
                               r'( +<(?P<email>.*)>)?'
                               r'( +\((?P<url>.*)\))?'
                               r' *$')

    def normalize_author(self, d):
        'https://docs.npmjs.com/files/package.json' \
                '#people-fields-author-contributors'
        author = {'@type': SCHEMA_URI+'Person'}
        if isinstance(d, dict):
            name = d.get('name', None)
            email = d.get('email', None)
            url = d.get('url', None)
        elif isinstance(d, str):
            match = self._parse_author.match(d)
            name = match.group('name')
            email = match.group('email')
            url = match.group('url')
        else:
            return None
        if name:
            author[SCHEMA_URI+'name'] = name
        if email:
            author[SCHEMA_URI+'email'] = email
        if url:
            author[SCHEMA_URI+'url'] = {'@id': url}
        return {"@list": [author]}

    def normalize_license(self, s):
        return {"@id": "https://spdx.org/licenses/" + s}

    def normalize_homepage(self, s):
        return {"@id": s}


@register_mapping
class CodemetaMapping(SingleFileMapping):
    """
    dedicated class for CodeMeta (codemeta.json) mapping and translation
    """
    filename = b'codemeta.json'

    def translate(self, content):
        return self.normalize_translation(expand(json.loads(content.decode())))


@register_mapping
class MavenMapping(DictMapping, SingleFileMapping):
    """
    dedicated class for Maven (pom.xml) mapping and translation
    """
    filename = b'pom.xml'
    mapping = CROSSWALK_TABLE['Java (Maven)']

    def translate(self, content):
        d = xmltodict.parse(content)['project']
        metadata = self.translate_dict(d, normalize=False)
        metadata[SCHEMA_URI+'codeRepository'] = self.parse_repositories(d)
        metadata[SCHEMA_URI+'license'] = self.parse_licenses(d)
        return self.normalize_translation(metadata)

    _default_repository = {'url': 'https://repo.maven.apache.org/maven2/'}

    def parse_repositories(self, d):
        """https://maven.apache.org/pom.html#Repositories"""
        if 'repositories' not in d:
            return [self.parse_repository(d, self._default_repository)]
        else:
            repositories = d['repositories'].get('repository', [])
            if not isinstance(repositories, list):
                repositories = [repositories]
            results = []
            for repo in repositories:
                res = self.parse_repository(d, repo)
                if res:
                    results.append(res)
            return results

    def parse_repository(self, d, repo):
        if repo.get('layout', 'default') != 'default':
            return  # TODO ?
        url = repo['url']
        if d['groupId']:
            url = os.path.join(url, *d['groupId'].split('.'))
            if d['artifactId']:
                url = os.path.join(url, d['artifactId'])
        return {"@id": url}

    def normalize_groupId(self, id_):
        return {"@id": id_}

    def parse_licenses(self, d):
        """https://maven.apache.org/pom.html#Licenses

        The origin XML has the form:

            <licenses>
              <license>
                <name>Apache License, Version 2.0</name>
                <url>https://www.apache.org/licenses/LICENSE-2.0.txt</url>
              </license>
            </licenses>

        Which was translated to a dict by xmltodict and is given as `d`:

        >>> d = {
        ...     # ...
        ...     "licenses": {
        ...         "license": {
        ...             "name": "Apache License, Version 2.0",
        ...             "url":
        ...             "https://www.apache.org/licenses/LICENSE-2.0.txt"
        ...         }
        ...     }
        ... }
        >>> MavenMapping().parse_licenses(d)
        [{'@id': 'https://www.apache.org/licenses/LICENSE-2.0.txt'}]

        or, if there are more than one license:

        >>> from pprint import pprint
        >>> d = {
        ...     # ...
        ...     "licenses": {
        ...         "license": [
        ...             {
        ...                 "name": "Apache License, Version 2.0",
        ...                 "url":
        ...                 "https://www.apache.org/licenses/LICENSE-2.0.txt"
        ...             },
        ...             {
        ...                 "name": "MIT License, ",
        ...                 "url": "https://opensource.org/licenses/MIT"
        ...             }
        ...         ]
        ...     }
        ... }
        >>> pprint(MavenMapping().parse_licenses(d))
        [{'@id': 'https://www.apache.org/licenses/LICENSE-2.0.txt'},
         {'@id': 'https://opensource.org/licenses/MIT'}]
        """

        licenses = d.get('licenses', {}).get('license', [])
        if isinstance(licenses, dict):
            licenses = [licenses]
        return [{"@id": license['url']} for license in licenses]


_normalize_pkginfo_key = str.lower


@register_mapping
class PythonPkginfoMapping(DictMapping, SingleFileMapping):
    """Dedicated class for Python's PKG-INFO mapping and translation.

    https://www.python.org/dev/peps/pep-0314/"""
    filename = b'PKG-INFO'
    mapping = {_normalize_pkginfo_key(k): v
               for (k, v) in CROSSWALK_TABLE['Python PKG-INFO'].items()}

    _parser = email.parser.BytesHeaderParser()

    def translate(self, content):
        msg = self._parser.parsebytes(content)
        d = {}
        for (key, value) in msg.items():
            key = _normalize_pkginfo_key(key)
            if value != 'UNKNOWN':
                d.setdefault(key, []).append(value)
        metadata = self.translate_dict(d, normalize=False)
        if SCHEMA_URI+'author' in metadata or SCHEMA_URI+'email' in metadata:
            metadata[SCHEMA_URI+'author'] = {
                '@list': [{
                    '@type': SCHEMA_URI+'Person',
                    SCHEMA_URI+'name':
                        metadata.pop(SCHEMA_URI+'author', [None])[0],
                    SCHEMA_URI+'email':
                        metadata.pop(SCHEMA_URI+'email', [None])[0],
                }]
            }
        return self.normalize_translation(metadata)

    def translate_summary(self, translated_metadata, v):
        k = self.mapping['summary']
        translated_metadata.setdefault(k, []).append(v)

    def translate_description(self, translated_metadata, v):
        k = self.mapping['description']
        translated_metadata.setdefault(k, []).append(v)

    def normalize_home_page(self, urls):
        return [{'@id': url} for url in urls]

    def normalize_license(self, licenses):
        return [{'@id': license} for license in licenses]


def main():
    raw_content = """{"name": "test_name", "unknown_term": "ut"}"""
    raw_content1 = b"""{"name": "test_name",
                        "unknown_term": "ut",
                        "prerequisites" :"packageXYZ"}"""
    result = MAPPINGS["NpmMapping"].translate(raw_content)
    result1 = MAPPINGS["MavenMapping"].translate(raw_content1)

    print(result)
    print(result1)


if __name__ == "__main__":
    main()



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import unittest
from swh.indexer import language
from swh.indexer.language import LanguageIndexer
from swh.indexer.tests.test_utils import (
    CommonContentIndexerTest, CommonIndexerWithErrorsTest,
    CommonIndexerNoTool, BASE_TEST_CONFIG, fill_storage, fill_obj_storage
)


class LanguageTestIndexer(LanguageIndexer):
    """Specific language whose configuration is enough to satisfy the
       indexing tests.
    """
    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'tools':  {
                'name': 'pygments',
                'version': '2.0.1+dfsg-1.1+deb8u1',
                'configuration': {
                    'type': 'library',
                    'debian-package': 'python3-pygments',
                    'max_content_size': 10240,
                },
            }
        }


class Language(unittest.TestCase):
    """Tests pygments tool for language detection

    """
    def test_compute_language_none(self):
        # given
        self.content = ""
        self.declared_language = {
            'lang': None
        }
        # when
        result = language.compute_language(self.content)
        # then
        self.assertEqual(self.declared_language, result)


class TestLanguageIndexer(CommonContentIndexerTest, unittest.TestCase):
    """Language indexer test scenarios:

    - Known sha1s in the input list have their data indexed
    - Unknown sha1 in the input list are not indexed

    """

    legacy_get_format = True

    def get_indexer_results(self, ids):
        yield from self.indexer.idx_storage.content_language_get(ids)

    def setUp(self):
        self.indexer = LanguageTestIndexer()
        fill_storage(self.indexer.storage)
        fill_obj_storage(self.indexer.objstorage)

        self.id0 = '02fb2c89e14f7fab46701478c83779c7beb7b069'
        self.id1 = '103bc087db1d26afc3a0283f38663d081e9b01e6'
        self.id2 = 'd4c647f0fc257591cc9ba1722484229780d1c607'

        tool = {k.replace('tool_', ''): v
                for (k, v) in self.indexer.tool.items()}

        self.expected_results = {
            self.id0: {
                'id': self.id0,
                'tool': tool,
                'lang': 'python',
            },
            self.id1: {
                'id': self.id1,
                'tool': tool,
                'lang': 'c'
            },
            self.id2: {
                'id': self.id2,
                'tool': tool,
                'lang': 'text-only'
            }
        }


class LanguageIndexerUnknownToolTestStorage(
        CommonIndexerNoTool, LanguageTestIndexer):
    """Fossology license indexer with wrong configuration"""


class TestLanguageIndexersErrors(
        CommonIndexerWithErrorsTest, unittest.TestCase):
    """Test the indexer raise the right errors when wrongly initialized"""
    Indexer = LanguageIndexerUnknownToolTestStorage



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import click
import itertools
import logging

from swh.indexer.indexer import ContentIndexer, RevisionIndexer, OriginIndexer
from swh.indexer.metadata_dictionary import MAPPINGS
from swh.indexer.metadata_detector import detect_metadata
from swh.indexer.metadata_detector import extract_minimal_metadata_dict
from swh.indexer.storage import INDEXER_CFG_KEY

from swh.model import hashutil


class ContentMetadataIndexer(ContentIndexer):
    """Content-level indexer

    This indexer is in charge of:

    - filtering out content already indexed in content_metadata
    - reading content from objstorage with the content's id sha1
    - computing translated_metadata by given context
    - using the metadata_dictionary as the 'swh-metadata-translator' tool
    - store result in content_metadata table

    """
    # Note: This used when the content metadata indexer is used alone
    # (not the case for example in the case of the RevisionMetadataIndexer)
    CONFIG_BASE_FILENAME = 'indexer/content_metadata'

    def __init__(self, tool, config):
        # FIXME: Simplify this twisted way to use the exact same
        # config of RevisionMetadataIndexer object that uses
        # internally ContentMetadataIndexer
        self.config = config
        self.config['tools'] = tool
        self.results = []
        super().__init__()
        self.tool = self.tools[0]  # Tool is now registered (cf. prepare call)

    def filter(self, ids):
        """Filter out known sha1s and return only missing ones.
        """
        yield from self.idx_storage.content_metadata_missing((
            {
                'id': sha1,
                'indexer_configuration_id': self.tool['id'],
            } for sha1 in ids
        ))

    def index(self, id, data):
        """Index sha1s' content and store result.

        Args:
            id (bytes): content's identifier
            data (bytes): raw content in bytes

        Returns:
            dict: dictionary representing a content_metadata. If the
            translation wasn't successful the translated_metadata keys will
            be returned as None

        """
        result = {
            'id': id,
            'indexer_configuration_id': self.tool['id'],
            'translated_metadata': None
        }
        try:
            mapping_name = self.tool['tool_configuration']['context']
            result['translated_metadata'] = MAPPINGS[mapping_name] \
                .translate(data)
        except Exception:
            self.log.exception(
                "Problem during tool retrieval of metadata translation")
        return result

    def persist_index_computations(self, results, policy_update):
        """Persist the results in storage.

        Args:
            results ([dict]): list of content_metadata, dict with the
              following keys:
              - id (bytes): content's identifier (sha1)
              - translated_metadata (jsonb): detected metadata
            policy_update ([str]): either 'update-dups' or 'ignore-dups' to
              respectively update duplicates or ignore them

        """
        self.idx_storage.content_metadata_add(
            results, conflict_update=(policy_update == 'update-dups'))


class RevisionMetadataIndexer(RevisionIndexer):
    """Revision-level indexer

    This indexer is in charge of:

    - filtering revisions already indexed in revision_metadata table with
      defined computation tool
    - retrieve all entry_files in root directory
    - use metadata_detector for file_names containing metadata
    - compute metadata translation if necessary and possible (depends on tool)
    - send sha1s to content indexing if possible
    - store the results for revision

    """
    CONFIG_BASE_FILENAME = 'indexer/revision_metadata'

    ADDITIONAL_CONFIG = {
        'tools': ('dict', {
            'name': 'swh-metadata-detector',
            'version': '0.0.2',
            'configuration': {
                'type': 'local',
                'context': ['NpmMapping', 'CodemetaMapping']
            },
        }),
    }

    ContentMetadataIndexer = ContentMetadataIndexer

    def prepare(self):
        super().prepare()
        self.tool = self.tools[0]

    def filter(self, sha1_gits):
        """Filter out known sha1s and return only missing ones.

        """
        yield from self.idx_storage.revision_metadata_missing((
            {
                'id': sha1_git,
                'indexer_configuration_id': self.tool['id'],
            } for sha1_git in sha1_gits
        ))

    def index(self, rev):
        """Index rev by processing it and organizing result.

        use metadata_detector to iterate on filenames

        - if one filename detected -> sends file to content indexer
        - if multiple file detected -> translation needed at revision level

        Args:
          rev (bytes): revision artifact from storage

        Returns:
            dict: dictionary representing a revision_metadata, with keys:

            - id (str): rev's identifier (sha1_git)
            - indexer_configuration_id (bytes): tool used
            - translated_metadata: dict of retrieved metadata

        """
        result = {
            'id': rev['id'],
            'indexer_configuration_id': self.tool['id'],
            'translated_metadata': None
        }

        try:
            root_dir = rev['directory']
            dir_ls = self.storage.directory_ls(root_dir, recursive=False)
            files = [entry for entry in dir_ls if entry['type'] == 'file']
            detected_files = detect_metadata(files)
            result['translated_metadata'] = self.translate_revision_metadata(
                    detected_files)
        except Exception as e:
            self.log.exception(
                'Problem when indexing rev: %r', e)
        return result

    def persist_index_computations(self, results, policy_update):
        """Persist the results in storage.

        Args:
            results ([dict]): list of content_mimetype, dict with the
              following keys:
              - id (bytes): content's identifier (sha1)
              - mimetype (bytes): mimetype in bytes
              - encoding (bytes): encoding in bytes
            policy_update ([str]): either 'update-dups' or 'ignore-dups' to
              respectively update duplicates or ignore them

        """
        # TODO: add functions in storage to keep data in revision_metadata
        self.idx_storage.revision_metadata_add(
            results, conflict_update=(policy_update == 'update-dups'))

    def translate_revision_metadata(self, detected_files):
        """
        Determine plan of action to translate metadata when containing
        one or multiple detected files:

        Args:
            detected_files (dict): dictionary mapping context names (e.g.,
              "npm", "authors") to list of sha1

        Returns:
            dict: dict with translated metadata according to the CodeMeta
            vocabulary

        """
        translated_metadata = []
        tool = {
                'name': 'swh-metadata-translator',
                'version': '0.0.2',
                'configuration': {
                    'type': 'local',
                    'context': None
                },
            }
        # TODO: iterate on each context, on each file
        # -> get raw_contents
        # -> translate each content
        config = {
            k: self.config[k]
            for k in [INDEXER_CFG_KEY, 'objstorage', 'storage']
        }
        for context in detected_files.keys():
            tool['configuration']['context'] = context
            c_metadata_indexer = self.ContentMetadataIndexer(tool, config)
            # sha1s that are in content_metadata table
            sha1s_in_storage = []
            metadata_generator = self.idx_storage.content_metadata_get(
                detected_files[context])
            for c in metadata_generator:
                # extracting translated_metadata
                sha1 = c['id']
                sha1s_in_storage.append(sha1)
                local_metadata = c['translated_metadata']
                # local metadata is aggregated
                if local_metadata:
                    translated_metadata.append(local_metadata)

            sha1s_filtered = [item for item in detected_files[context]
                              if item not in sha1s_in_storage]

            if sha1s_filtered:
                # content indexing
                try:
                    c_metadata_indexer.run(sha1s_filtered,
                                           policy_update='ignore-dups')
                    # on the fly possibility:
                    for result in c_metadata_indexer.results:
                        local_metadata = result['translated_metadata']
                        translated_metadata.append(local_metadata)

                except Exception:
                    self.log.exception(
                        "Exception while indexing metadata on contents")

        # transform translated_metadata into min set with swh-metadata-detector
        min_metadata = extract_minimal_metadata_dict(translated_metadata)
        return min_metadata


class OriginMetadataIndexer(OriginIndexer):
    CONFIG_BASE_FILENAME = 'indexer/origin_intrinsic_metadata'

    ADDITIONAL_CONFIG = {
        'tools': ('list', [])
    }

    def check(self, **kwargs):
        kwargs['check_tools'] = False
        super().check(**kwargs)

    def filter(self, ids):
        return ids

    def run(self, origin_head, policy_update):
        """Expected to be called with the result of RevisionMetadataIndexer
        as first argument; ie. not a list of ids as other indexers would.

        Args:
            origin_head (dict): {str(origin_id): rev_id}
              keys `origin_id` and `revision_id`, which is the result
              of OriginHeadIndexer.
            policy_update (str): `'ignore-dups'` or `'update-dups'`
        """
        origin_head_map = {origin_id: hashutil.hash_to_bytes(rev_id)
                           for (origin_id, rev_id) in origin_head.items()}

        # Fix up the argument order. revisions_metadata has to be the
        # first argument because of celery.chain; the next line calls
        # run() with the usual order, ie. origin ids first.
        return super().run(ids=list(origin_head_map),
                           policy_update=policy_update,
                           parse_ids=False,
                           origin_head_map=origin_head_map)

    def index(self, origin, *, origin_head_map):
        # Get the last revision of the origin.
        revision_id = origin_head_map[str(origin['id'])]

        revision_metadata = self.idx_storage \
            .revision_metadata_get([revision_id])

        results = []
        for item in revision_metadata:
            assert item['id'] == revision_id
            # Get the metadata of that revision, and return it
            results.append({
                    'origin_id': origin['id'],
                    'metadata': item['translated_metadata'],
                    'from_revision': revision_id,
                    'indexer_configuration_id':
                    item['tool']['id'],
                    })
        return results

    def persist_index_computations(self, results, policy_update):
        self.idx_storage.origin_intrinsic_metadata_add(
            list(itertools.chain(*results)),
            conflict_update=(policy_update == 'update-dups'))


@click.command()
@click.option('--revs', '-i',
              help='Default sha1_git to lookup', multiple=True)
def main(revs):
    _git_sha1s = list(map(hashutil.hash_to_bytes, revs))
    rev_metadata_indexer = RevisionMetadataIndexer()
    rev_metadata_indexer.run(_git_sha1s, 'update-dups')


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    main()



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

import unittest

from unittest.mock import patch

from swh.indexer.mimetype import (
    MimetypeIndexer, MimetypeRangeIndexer, compute_mimetype_encoding
)

from swh.indexer.tests.test_utils import (
    CommonContentIndexerTest, CommonContentIndexerRangeTest,
    CommonIndexerWithErrorsTest, CommonIndexerNoTool,
    BASE_TEST_CONFIG, fill_storage, fill_obj_storage
)


class FakeMagicResult:
    def __init__(self, mimetype, encoding):
        self.mime_type = mimetype
        self.encoding = encoding


class BasicTest(unittest.TestCase):
    @patch('swh.indexer.mimetype.magic')
    def test_compute_mimetype_encoding(self, mock_magic):
        """Compute mimetype encoding should return results"""
        for _input, _mimetype, _encoding in [
                (b'some-content', 'text/plain', 'utf-8'),
                (b'raw-content', 'application/json', 'ascii')]:
            mock_magic.detect_from_content.return_value = FakeMagicResult(
                _mimetype, _encoding)

            actual_result = compute_mimetype_encoding(_input)
            self.assertEqual(actual_result, {
                'mimetype': _mimetype,
                'encoding': _encoding
            })


class MimetypeTestIndexer(MimetypeIndexer):
    """Specific mimetype indexer instance whose configuration is enough to
       satisfy the indexing tests.

    """
    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'tools': {
                'name': 'file',
                'version': '1:5.30-1+deb9u1',
                'configuration': {
                    "type": "library",
                    "debian-package": "python3-magic"
                },
            },
        }


class TestMimetypeIndexer(CommonContentIndexerTest, unittest.TestCase):
    """Mimetype indexer test scenarios:

    - Known sha1s in the input list have their data indexed
    - Unknown sha1 in the input list are not indexed

    """
    legacy_get_format = True

    def get_indexer_results(self, ids):
        yield from self.idx_storage.content_mimetype_get(ids)

    def setUp(self):
        self.indexer = MimetypeTestIndexer()
        self.idx_storage = self.indexer.idx_storage
        fill_storage(self.indexer.storage)
        fill_obj_storage(self.indexer.objstorage)

        self.id0 = '01c9379dfc33803963d07c1ccc748d3fe4c96bb5'
        self.id1 = '688a5ef812c53907562fe379d4b3851e69c7cb15'
        self.id2 = 'da39a3ee5e6b4b0d3255bfef95601890afd80709'

        tool = {k.replace('tool_', ''): v
                for (k, v) in self.indexer.tool.items()}

        self.expected_results = {
            self.id0: {
                'id': self.id0,
                'tool': tool,
                'mimetype': 'text/plain',
                'encoding': 'us-ascii',
            },
            self.id1: {
                'id': self.id1,
                'tool': tool,
                'mimetype': 'text/plain',
                'encoding': 'us-ascii',
            },
            self.id2: {
                'id': self.id2,
                'tool': tool,
                'mimetype': 'application/x-empty',
                'encoding': 'binary',
            }
        }


class MimetypeRangeIndexerTest(MimetypeRangeIndexer):
    """Specific mimetype whose configuration is enough to satisfy the
       indexing tests.

    """
    def parse_config_file(self, *args, **kwargs):
        return {
            **BASE_TEST_CONFIG,
            'tools': {
                'name': 'file',
                'version': '1:5.30-1+deb9u1',
                'configuration': {
                    "type": "library",
                    "debian-package": "python3-magic"
                },
            },
            'write_batch_size': 100,
        }


class TestMimetypeRangeIndexer(
        CommonContentIndexerRangeTest, unittest.TestCase):
    """Range Mimetype Indexer tests.

    - new data within range are indexed
    - no data outside a range are indexed
    - with filtering existing indexed data prior to compute new index
    - without filtering existing indexed data prior to compute new index

    """
    def setUp(self):
        super().setUp()
        self.indexer = MimetypeRangeIndexerTest()
        fill_storage(self.indexer.storage)
        fill_obj_storage(self.indexer.objstorage)

        self.id0 = '01c9379dfc33803963d07c1ccc748d3fe4c96bb5'
        self.id1 = '02fb2c89e14f7fab46701478c83779c7beb7b069'
        self.id2 = '103bc087db1d26afc3a0283f38663d081e9b01e6'
        tool_id = self.indexer.tool['id']

        self.expected_results = {
            self.id0: {
                'encoding': 'us-ascii',
                'id': self.id0,
                'indexer_configuration_id': tool_id,
                'mimetype': 'text/plain'},
            self.id1: {
                'encoding': 'us-ascii',
                'id': self.id1,
                'indexer_configuration_id': tool_id,
                'mimetype': 'text/x-python'},
            self.id2: {
                'encoding': 'us-ascii',
                'id': self.id2,
                'indexer_configuration_id': tool_id,
                'mimetype': 'text/plain'}
        }


class MimetypeIndexerUnknownToolTestStorage(
        CommonIndexerNoTool, MimetypeTestIndexer):
    """Mimetype indexer with wrong configuration"""


class MimetypeRangeIndexerUnknownToolTestStorage(
        CommonIndexerNoTool, MimetypeRangeIndexerTest):
    """Mimetype range indexer with wrong configuration"""


class TestMimetypeIndexersErrors(
        CommonIndexerWithErrorsTest, unittest.TestCase):
    """Test the indexer raise the right errors when wrongly initialized"""
    Indexer = MimetypeIndexerUnknownToolTestStorage
    RangeIndexer = MimetypeRangeIndexerUnknownToolTestStorage
